unit zitate_server;

{$mode objfpc}{$H+}
{$I winzitat_ver.inc}

interface

uses
   Classes, SysUtils;

{ TZitadell }

const
   daten_slash = {$IFDEF Linux} '/' {$ELSE} '\' {$ENDIF};

type
   TZitadell = class(TObject)
      constructor create;
      function Str0(n, anz: integer): string;
      function Parameter_Info: string;
      procedure Hole_Spruch(lesen: boolean);
      procedure SpruchEingeben;
      procedure SpruchLetztes;
      procedure SpruchZufall;
      procedure AbfolgeVor;
      procedure AbfolgeZurueck;
      procedure AbfolgeBeideZufall;
      procedure AbfolgeBeidesLetztes;
      procedure DateiEingeben;
      procedure DateiLetztes;
      procedure DateiZufall;
   private
      procedure Vorlauf;
      procedure Lies_Zeile1;
      procedure Zaehle_Sprueche;
      procedure Suche_Spruch;
      procedure TestSatz(var sinc,sdec: longint);
      {$IFNDEF Linux}
      function GetSpecialFolder(Folder: Integer): String;
      function GetHomeZitadell:string;
      function GetRegZitadell:string;
      {$ENDIF}
      function GetUserDirZitadell:string;
      function GetEnvZitadell:string;
   public
      spruch_text:                                                string;
      dauer, anzahl,
      spruch_nr, spruch_akt, spruch_max, spruch_min,
      datei_nr, datei_akt, datei_max, datei_min:                 longint;
      fio:                                                       boolean;
   private
      spruch_tmp, spruch_bak,
      datei_tmp, datei_bak,
      zitat_nr, zitat_zeile1_nr, zitat_anz, zitat_vor,
      zitat_abfolge, zitat_zwi, zitat_ext_von, zitat_ext_bis:    longint;
      datei_einm, spruch_einm,
      ok, zitat_zeile1_da, zitat_gefunden:                       boolean;
      zitat_zeile,
      daten_datei_name, daten_datei_ext, daten_pfad:              string;
      zitat_datei:                                                  text;
   end;

var
   Zitadell: TZitadell;

implementation

uses dos {$IFNDEF Linux}, shlobj, registry {$ENDIF}, Box, LConvEncoding;

constructor TZitadell.Create;
begin
   TObject.Create;
   fio:=false; spruch_bak:=0; datei_bak:=0;
   spruch_tmp:=1; datei_tmp:=1; spruch_einm:=true;
   datei_nr:=0; spruch_nr:=0; zitat_abfolge:=0;
   dauer:=5; anzahl:=10;
end;

function TZitadell.Str0(n, anz: integer): string;
var s0, s1: string; i: integer;
begin
   STR(n, s0); s1:='';
   for i:= anz - LENGTH(s0) downto 1 do s1:= s1 + '0';
   Result:= s1 + s0;
end;

{$IFDEF ZitadellCodeOEM}
function Zitadell_Filter_OEM(filstr: string): string;
begin
   Result:= Cp850ToUtf8(filstr);
end;
{$ENDIF}

{$IFDEF ZitadellCodeANSI}
function Zitadell_Filter_ANSI(filstr: string): string;
begin
   Result:= Cp1252ToUtf8(filstr);
end;
{$ENDIF}

{$IFDEF ZitadellCodeEIGEN}
function Zitadell_Filter_EIGEN(filstr: string): string;
var i0: longint; temstr: string; b0: boolean; z0:char; c0: integer;
begin
   temstr:=''; b0:=true;
   for i0:=1 TO LENGTH(filstr) do begin
      z0:=filstr[i0]; c0:=ORD(z0);
      if c0 <> 32 then b0:=false;
      case c0 of
         32: if not b0 then temstr:=temstr + ' ';
        132: temstr:=temstr + 'ä';   148: temstr:=temstr + 'ö';
        129: temstr:=temstr + 'ü';   142: temstr:=temstr + 'Ä';
        153: temstr:=temstr + 'Ö';   154: temstr:=temstr + 'Ü';
        225: temstr:=temstr + 'ß';   else temstr:=temstr + z0
      end;
   end;
   Result:= temstr;
end;
{$ENDIF}
{ Pfad... }

{$IFNDEF Linux}
function TZitadell.GetSpecialFolder(Folder: Integer): String;
begin
  SetLength(Result,MAX_PATH);
  If SHGetSpecialFolderPath(0, PChar(Result), Folder, false)
    then SetLength(Result,StrLen(PChar(Result)))
    else Result:='';
end;

function TZitadell.GetHomeZitadell:string;
var tempdir: string;
begin
   tempdir:=GetSpecialFolder(CSIDL_PERSONAL) + daten_slash + 'zitadell';
   if DirectoryExists(tempdir) and FileExists(tempdir + daten_slash + 'zitate.001')
   then result:=tempdir else result:='';
end;

function TZitadell.GetRegZitadell:string;
var Registry: TRegistry;
    tempdir: string;
begin
   Registry := TRegistry.Create; tempdir:='';
   try
      Registry.RootKey := HKEY_LOCAL_MACHINE;
      if Registry.OpenKeyReadOnly('\SOFTWARE\Alfware\Zitadell')
         then tempdir:= Registry.ReadString('');
   finally
      Registry.Free;
   end;
   if DirectoryExists(tempdir) and FileExists(tempdir + daten_slash + 'zitate.001')
   then result:=tempdir else result:='';
end;
{$ENDIF}

function TZitadell.GetUserDirZitadell:string;
var tempdir:string;
begin
   tempdir:=GetUserDir + 'documents' + daten_slash + 'zitadell';
   if DirectoryExists(tempdir) and FileExists(tempdir + daten_slash + 'zitate.001')
   then result:=tempdir else result:='';
end;

function TZitadell.GetEnvZitadell:string;
var tempdir:string;
begin
   tempdir:=GetEnv('ZITADELL');
   if DirectoryExists(tempdir) and FileExists(tempdir + daten_slash + 'zitate.001')
   then result:=tempdir else result:='';
end;

{ Zitate }

procedure TZitadell.TestSatz(var sinc,sdec: longint);
begin
   ReadLN(zitat_datei, zitat_zeile);
   if (COPY(zitat_zeile, 1, 3) = '###') or EOF(zitat_datei) then begin
      sinc:=sinc + 1; sdec:=sdec - 1
   end;
end;

procedure TZitadell.Vorlauf;
begin
   zitat_anz:=0; zitat_nr:=0; datei_akt:=0;
   ok:=false; zitat_zeile1_da:=false; zitat_gefunden:=false;
   Randomize;
   spruch_text:=''; daten_datei_name:=''; daten_pfad:='';
   zitat_ext_von:=datei_min;
   zitat_ext_bis:=datei_max;
   if zitat_ext_bis > zitat_ext_von
      then zitat_ext_bis:=zitat_ext_bis - zitat_ext_von
      else zitat_ext_bis:= -1;
   if datei_nr <> 0
      then datei_akt:=datei_nr
      else datei_akt:=RANDOM(zitat_ext_bis + 1) + zitat_ext_von;
   if datei_nr <> 0 then datei_nr:=datei_akt;
   daten_datei_ext:='.' + Str0(datei_akt, 3);
   (* Umgebungsvariable *)
   daten_pfad:=GetEnvZitadell;
   {$IFNDEF Linux}
   (* Registry *)
   if daten_pfad = '' then daten_pfad:= GetRegZitadell;
   (* Home-Dir 1.Versuch *)
   if daten_pfad = '' then daten_pfad:= GetHomeZitadell;
   {$ENDIF}
   (* Home-Dir 2.Versuch *)
   if daten_pfad = '' then daten_pfad:= GetUserDirZitadell;
   if daten_pfad <> '' then daten_pfad:= daten_pfad + daten_slash;
   daten_datei_name:= daten_pfad + 'zitate' + daten_datei_ext;
end;

procedure TZitadell.Lies_Zeile1;
var w: word;
begin
   ReadLN(zitat_datei, zitat_zeile);
   if (zitat_zeile[1] = '~') and (zitat_zeile[ LENGTH(zitat_zeile) ] = '~') then begin
      zitat_zeile1_da:=true; w:=99;
      VAL( COPY(zitat_zeile, 2, LENGTH(zitat_zeile)- 2 ), zitat_zeile1_nr, w);
      if w = 0 then begin
         if zitat_zeile1_nr > 0 then zitat_anz:=zitat_zeile1_nr;
         if zitat_zeile1_nr = 0 then zitat_gefunden:=true;
      end
   end;
end;

procedure TZitadell.Zaehle_Sprueche;
begin
   Reset(zitat_datei);
   while (not EOF(zitat_datei) or ok) do begin
      TestSatz(zitat_anz, zitat_zwi);
      if (not zitat_gefunden) and (zitat_nr>0) then ok:=(zitat_nr = zitat_anz);
   end;
end;

procedure TZitadell.Suche_Spruch;
begin
   spruch_text:='';
   if spruch_nr <> 0 then begin
      zitat_nr:=spruch_nr + zitat_abfolge;
      if zitat_nr > spruch_max then zitat_nr:=1;
      if zitat_nr < 1 then zitat_nr:=spruch_max;
   end;
   if (zitat_nr < 1) or (zitat_nr > zitat_anz)
      then zitat_vor:=RANDOM(zitat_anz)
      else zitat_vor:=zitat_nr - 1;
   if zitat_gefunden then zitat_vor:=zitat_anz;
   spruch_akt:=zitat_vor + 1;
   if spruch_nr <> 0 then spruch_nr:=spruch_akt;
   Reset(zitat_datei);
   if zitat_zeile1_da then ReadLN(zitat_datei, zitat_zeile);
   while (zitat_vor > 0) and not EOF(zitat_datei) do TestSatz(zitat_zwi, zitat_vor);
   if not (EOF(zitat_datei) or zitat_gefunden) then begin
      repeat
         ReadLN(zitat_datei, zitat_zeile);
         if COPY(zitat_zeile, 1, 3) <> '###' then begin
            {$IFDEF ZitadellCodeOEM}   zitat_zeile:=Zitadell_Filter_OEM(zitat_zeile);   {$ENDIF}
            {$IFDEF ZitadellCodeANSI}  zitat_zeile:=Zitadell_Filter_ANSI(zitat_zeile);  {$ENDIF}
            {$IFDEF ZitadellCodeEIGEN} zitat_zeile:=Zitadell_Filter_EIGEN(zitat_zeile); {$ENDIF}
            spruch_text:=spruch_text + zitat_zeile + slinebreak;
         end;
      until EOF(zitat_datei) or (COPY(zitat_zeile, 1, 3) = '###');
   end;
   fio:=true;
end;

procedure TZitadell.Hole_Spruch(lesen: boolean);
begin
   Vorlauf;
   AssignFile(zitat_datei, daten_datei_name);
   if FileExists(daten_datei_name) then begin
      Reset(zitat_datei);
      Lies_Zeile1;
      if zitat_anz = 0 then Zaehle_Sprueche;
      if lesen
         then begin if zitat_anz > 0 then Suche_Spruch; end
         else spruch_max:=zitat_anz;
      Close(zitat_datei);
   end;
   if lesen then begin
      if spruch_text = '' then spruch_text:='Leider keine bekannt...' + slinebreak;
   end;
end;

function TZitadell.Parameter_Info: string;
var datei_aus, spruch_aus: longint;
    info: string;
begin
   if datei_nr = 0
      then datei_aus:=datei_akt
      else datei_aus:=datei_nr;
   if datei_nr = 0
      then info:=string('(zufällige) ')
      else info:='';
   info:=info + string(' Sprüche  d a t e i : ') + Str0(datei_aus,2) + slinebreak + slinebreak;
   if spruch_nr = 0
      then spruch_aus:=spruch_akt
      else spruch_aus:=spruch_nr;
   if spruch_nr = 0
      then info:=info + string('zufälliger')
      else info:=info + string('ausgewählter');
   Result:=info + '  S p r u c h : ' + Str0(spruch_aus,3) + slinebreak;
end;

{ TForm1 verarbeiten }

procedure TZitadell.SpruchEingeben;
begin
   Form3.Caption:='Spruch';
   Form3.min:=1;
   Form3.max:=spruch_max;
   Form3.wert:=spruch_akt;
   Form3.ShowModal;
   spruch_tmp:=Form3.wert;
   if spruch_einm then begin
      spruch_einm:=false;
      spruch_bak:=spruch_nr;
   end;
   if spruch_tmp > spruch_max then spruch_tmp:=spruch_max;
   zitat_abfolge:=0;
   spruch_nr:=spruch_tmp;
end;

procedure TZitadell.SpruchLetztes;
begin
   fio:=false; zitat_abfolge:=0; spruch_tmp:=spruch_bak;
   if spruch_tmp <> spruch_nr then begin;
      if spruch_einm then begin;
         spruch_einm:=false;
         spruch_bak:=spruch_nr;
      end;
      spruch_bak:=spruch_nr;
      spruch_nr:=spruch_tmp;
   end;
end;

procedure TZitadell.SpruchZufall;
begin
   fio:=false; zitat_abfolge:=0; spruch_tmp:=0;
   if spruch_tmp <> spruch_nr then begin;
      if spruch_einm then begin;
         spruch_einm:=false;
         spruch_bak:=spruch_nr;
      end;
      spruch_bak:=spruch_nr;
      spruch_nr:=spruch_tmp;
   end;
end;

procedure TZitadell.AbfolgeVor;
begin
   fio:=false; zitat_abfolge:=1;
   datei_nr:=datei_akt;
   spruch_nr:=spruch_akt;
end;

procedure TZitadell.AbfolgeZurueck;
begin
   fio:=false; zitat_abfolge:=-1;
   datei_nr:=datei_akt;
   spruch_nr:=spruch_akt;
end;

procedure TZitadell.AbfolgeBeideZufall;
begin
   fio:=false; zitat_abfolge:=0; datei_tmp:=0;
   if datei_tmp <> datei_nr then begin;
      if datei_einm then begin;
         datei_einm:=false;
         datei_bak:=datei_nr;
       end;
       datei_bak:=datei_nr;
       datei_nr:=datei_tmp;
   end;
   spruch_tmp:=0;
   if spruch_tmp <> spruch_nr then begin;
      if spruch_einm then begin;
         spruch_einm:=false;
         spruch_bak:=spruch_nr;
      end;
      spruch_bak:=spruch_nr;
      spruch_nr:=spruch_tmp;
   end;
end;

procedure TZitadell.AbfolgeBeidesLetztes;
begin
   fio:=false; zitat_abfolge:=0; datei_tmp:=datei_bak;
   if datei_tmp <> datei_nr then begin;
      if datei_einm then begin;
         datei_einm:=false;
         datei_bak:=datei_nr;
      end;
      datei_bak:=datei_nr;
      datei_nr:=datei_tmp;
   end;
   spruch_tmp:=spruch_bak;
   if spruch_tmp <> spruch_nr then begin;
      if spruch_einm then begin;
         spruch_einm:=false;
         spruch_bak:=spruch_nr;
      end;
      spruch_bak:=spruch_nr;
      spruch_nr:=spruch_tmp;
   end;
end;

procedure TZitadell.DateiEingeben;
begin
   Form3.Caption:='Datei';
   Form3.min:=datei_min;
   Form3.max:=datei_max;
   Form3.wert:=datei_akt;
   Form3.ShowModal;
   datei_tmp:=Form3.wert;
   if datei_tmp <> datei_nr then begin;
      if datei_einm then begin;
         datei_einm:=false;
         datei_bak:=datei_nr;
      end;
      datei_nr:=datei_tmp;
      Hole_Spruch(false);
      { zitat_abfolge:=0; }
      if spruch_nr > spruch_max then spruch_nr:=spruch_max;
   end;
end;

procedure TZitadell.DateiLetztes;
begin
   fio:=false; zitat_abfolge:=0; datei_tmp:=datei_bak;
   if datei_tmp <> datei_nr then begin;
      if datei_einm then begin;
         datei_einm:=false;
         datei_bak:=datei_nr;
      end;
      datei_bak:=datei_nr; datei_nr:=datei_tmp;
      Hole_Spruch(false);
      if spruch_nr > spruch_max then spruch_nr:=spruch_max;
   end;
end;

procedure TZitadell.DateiZufall;
begin
   fio:=false; zitat_abfolge:=0; datei_tmp:=0;
   if datei_tmp <> datei_nr then begin;
      if datei_einm then begin;
         datei_einm:=false;
         datei_bak:=datei_nr;
      end;
      datei_bak:=datei_nr; datei_nr:=datei_tmp;
      Hole_Spruch(false);
      if spruch_nr > spruch_max then spruch_nr:=spruch_max;
   end;
end;

begin
   Randomize;
   Zitadell:=TZitadell.Create;
end.
